#include <QPixmap>
#include <QMenu>
#include <QUrl>
#include <QClipboard>
#include <QScriptEngine>
#include <QTextStream>
#include <QFile>
#include <QDir>
#include <QDebug>

#include "hasherthread.h"
#include "mainview.h"
#include "ui_mainview.h"

static const QString versionString( "0.5.0" );
static const QString startupText( "Tool Box %1\n"
                                  "Copyright 2009-2010 Richard Moore, rich@sectools.co.uk\n"
                                  "\n" );

MainView::MainView(QWidget *parent) :
    QTabWidget(parent),
    ui(new Ui::MainView)
{
    ui->setupUi(this);
    setupInterpreter();

    createTrayIcon();
    createAsciiTable();
    createScriptPage();
    createLookupPage();
    setupHashPage();

    connect( ui->urlEncodeButton, SIGNAL(clicked()), SLOT(urlEncodeText()) );
    connect( ui->urlDecodeButton, SIGNAL(clicked()), SLOT(urlDecodeText()) );
    connect( ui->base64EncodeButton, SIGNAL(clicked()), SLOT(base64EncodeText()) );
    connect( ui->base64DecodeButton, SIGNAL(clicked()), SLOT(base64DecodeText()) );
    connect( ui->hexEncodeButton, SIGNAL(clicked()), SLOT(hexEncodeText()) );
    connect( ui->hexDecodeButton, SIGNAL(clicked()), SLOT(hexDecodeText()) );
}

MainView::~MainView()
{
    delete ui;
}

QString MainView::findResource( const QString &filename )
{
    QDir appDir( QApplication::applicationDirPath() );

    // Hack to handle release and debug subdirs used by creator
    QString path = appDir.filePath( filename );
    if ( QFile(path).exists() )
        return path;
    else {
        appDir.cdUp();
        return appDir.filePath( filename );
    }
}

bool MainView::loadScript( const QString &filename )
{
    QFile f(filename);
    if (!f.open( QFile::ReadOnly ) ) {
        qDebug() << QString("Could not open file ") << filename;
        return false;
    }

    QTextStream ts( &f );
    QString script = ts.readAll();

    QScriptValue result = scriptEngine->evaluate( script );
    if ( scriptEngine->hasUncaughtException() ) {
        qDebug() << QString("Error parsing file ") << filename << result.toString();
        return false;
    }
    return true;
}

void MainView::setupInterpreter()
{
    scriptEngine = new QScriptEngine( this );

    QScriptValue global = scriptEngine->globalObject();
    global.setProperty( QString("version"), versionString,
                        QScriptValue::ReadOnly | QScriptValue::Undeletable );

    // Load builtins
    loadScript( findResource( QString("oui.js" ) ) );
    loadScript( findResource( QString("lookupmac.js" ) ) );
    loadScript( findResource( QString("checkip.js" ) ) );
    loadScript( findResource( QString("netutils.js" ) ) );

    ui->scriptOutputView->appendPlainText( startupText.arg(versionString) );
}

void MainView::createTrayIcon()
{
    QSystemTrayIcon *icon = new QSystemTrayIcon(this);

    QPixmap pix( 16, 16 );
    pix.fill( Qt::green );
    icon->setIcon( QIcon(pix) );
    icon->setToolTip( "Tool Box" );

    QMenu *menu = new QMenu( this );

    menu->addAction( QString("URL Encode Clipboard"), this, SLOT(urlEncodeClipboard()) );
    menu->addAction( QString("URL Decode Clipboard"), this, SLOT(urlDecodeClipboard()) );
    menu->addAction( QString("Lookup Clipboard"), this, SLOT(lookupClipboard()) );
    menu->addSeparator();
    menu->addAction( QString("Quit"), qApp, SLOT(quit()) );

    icon->setContextMenu( menu );

    connect( icon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
             SLOT(trayIconActivated(QSystemTrayIcon::ActivationReason)));
    icon->show();
}

void MainView::createAsciiTable()
{
    QTreeWidget *table = ui->asciiTable;

    for ( int i = 32; i < 127; i++ ) {
        QStringList cols;
        cols.append( QString::number(i) );
        cols.append( QString::number(i, 16) );
        cols.append( QString( QChar(i) ) );

        table->addTopLevelItem( new QTreeWidgetItem(cols) );
    }
}

void MainView::createScriptPage()
{
    connect( ui->scriptEntryEdit, SIGNAL(returnPressed()), SLOT(scriptEntryRun()) );
}

void MainView::createLookupPage()
{
    connect( ui->lookupButton, SIGNAL(clicked()), ui->thingEntry, SIGNAL(returnPressed()) );
    connect( ui->thingEntry, SIGNAL(returnPressed()), SLOT(lookup()) );
}

void MainView::closeEvent( QCloseEvent *ev )
{
    ev->ignore();
    hide();
}

void MainView::trayIconActivated( QSystemTrayIcon::ActivationReason reason )
{
    if ( reason == QSystemTrayIcon::Context )
        return;

    if ( this->isVisible() )
        hide();
    else {
        show();
        activateWindow();
    }
}

void MainView::urlEncodeClipboard()
{
    QString text = qApp->clipboard()->text();
    text = QUrl::toPercentEncoding( text );
    qApp->clipboard()->setText( text );
}

void MainView::urlDecodeClipboard()
{
    QString text = qApp->clipboard()->text();
    text = QUrl::fromPercentEncoding( text.toUtf8() );
    qApp->clipboard()->setText( text );
}

void MainView::urlEncodeText()
{
    QString text = ui->plainTextEdit->toPlainText();
    text = QUrl::toPercentEncoding( text );
    ui->plainTextEdit->setPlainText( text );
}

void MainView::urlDecodeText()
{
    QString text = ui->plainTextEdit->toPlainText();
    text = QUrl::fromPercentEncoding( text.toUtf8() );
    ui->plainTextEdit->setPlainText( text );
}

void MainView::base64EncodeText()
{
    QString text = ui->plainTextEdit->toPlainText();
    text = QString( text.toUtf8().toBase64() );
    ui->plainTextEdit->setPlainText( text );
}

void MainView::base64DecodeText()
{
    QString text = ui->plainTextEdit->toPlainText();
    text = QString( QByteArray::fromBase64( text.toUtf8() ) );
    ui->plainTextEdit->setPlainText( text );
}

void MainView::hexEncodeText()
{
    QString text = ui->plainTextEdit->toPlainText();
    text = QString( text.toUtf8().toHex() );
    ui->plainTextEdit->setPlainText( text );
}

void MainView::hexDecodeText()
{
    QString text = ui->plainTextEdit->toPlainText();
    text = QString( QByteArray::fromHex( text.toUtf8() ) );
    ui->plainTextEdit->setPlainText( text );
}

void MainView::scriptEntryRun()
{
    QString script = ui->scriptEntryEdit->text();
    ui->scriptOutputView->appendPlainText( QString(">>> %1").arg(script) );

    QScriptValue result = scriptEngine->evaluate( script );
    ui->scriptOutputView->appendPlainText( result.toString() );
}

void MainView::lookup()
{
    lookup( ui->thingEntry->text() );
}

void MainView::lookupClipboard()
{
    QString text = qApp->clipboard()->text();
    ui->thingEntry->setText( text );
    lookup( text );

    setCurrentIndex(2);

    if ( !isVisible() ) {
        show();
        activateWindow();
    }
}

void MainView::lookup( const QString &thing )
{
    QRegExp cidr( "\\d+\\.\\d+\\.\\d+\\.\\d+/\\d+" );
    QRegExp ip( "\\d+\\.\\d+\\.\\d+\\.\\d+" );
    QRegExp mac( "[0-9a-fA-F][0-9a-fA-F][\\-:\\.][0-9a-fA-F][0-9a-fA-F][\\-:\\.][0-9a-fA-F][0-9a-fA-F]" );

    QString trimmed = thing.trimmed();
    if ( cidr.exactMatch( trimmed ) ) {
        lookupCIDR(trimmed);
    }
    else if ( ip.exactMatch( trimmed ) ) {
        lookupIpAddress(trimmed);
    }
    else if ( mac.indexIn(trimmed) != -1 ) {
        lookupMacAddress(trimmed);
    }
    else {
        lookupDomain(trimmed);
    }
}

void MainView::lookupIpAddress( const QString &ip )
{
    QScriptValue global = scriptEngine->globalObject();
    QScriptValue checkIp = global.property( QString("check_ip") );

    QScriptValueList args;
    args << ip;

    QScriptValue result = checkIp.call( QScriptValue(), args );
    ui->lookupView->setPlainText( QString("IP Address: %1\n%2").arg(ip).arg( result.toString() ) );
}

void MainView::lookupDomain( const QString &domain )
{
    QHostInfo::lookupHost( domain, this, SLOT(lookupDomainComplete( QHostInfo ) ) );
}

void MainView::lookupDomainComplete( QHostInfo info )
{
    if ( info.error() != QHostInfo::NoError ) {
        QString output( "Domain Name: %1\n\nDid not resolve:\n\n%2" );
        ui->lookupView->setPlainText( output.arg( info.hostName() ).arg( info.errorString() ) );
        return;
    }

    QString output( "Domain Name: %1\n\nResolves to:\n\n%2" );

    QString addresses;
    foreach( QHostAddress addr, info.addresses() ) {
        addresses += addr.toString();
        addresses += QString("\n");
    }

    ui->lookupView->setPlainText( output.arg( info.hostName() ).arg( addresses ) );
}

void MainView::lookupMacAddress( const QString &mac )
{
    QScriptValue global = scriptEngine->globalObject();
    QScriptValue lookup = global.property( QString("lookup_ethernet") );

    QScriptValueList args;
    args << mac;

    QScriptValue result = lookup.call( global, args );

    ui->lookupView->setPlainText( QString("MAC Address: %1\n\n%2").arg(mac).arg(result.toString()) );
}

void MainView::lookupCIDR( const QString &cidr )
{
    QStringList cidrList = cidr.split( QChar('/') );

    QScriptValue global = scriptEngine->globalObject();
    QScriptValue lookup = global.property( QString("cidr_info") );

    QScriptValueList args;
    args << cidrList[0];
    args << cidrList[1];

    QScriptValue result = lookup.call( global, args );
    ui->lookupView->setPlainText( QString("CIDR: %1\n\n%2").arg(cidr).arg(result.toString()) );
}

void MainView::setupHashPage()
{
    hasher = new HasherThread( this );
    connect( hasher, SIGNAL(error(const QString &)), ui->resultEdit, SLOT(setText(const QString &)) );
    connect( hasher, SIGNAL(completed(const QString &)), ui->resultEdit, SLOT(setText(const QString &)) );

    connect( ui->hashButton, SIGNAL(clicked()), SLOT(hashPressed()) );
}

void MainView::hashPressed()
{
    if ( ui->filenameEdit->text().isEmpty() )
        return;
    hasher->setFilename( ui->filenameEdit->text() );

    switch( ui->algorithmCombo->currentIndex() ) {
    case 0:
        hasher->setAlgorithm( QCryptographicHash::Sha1 );
        break;
    case 1:
        hasher->setAlgorithm( QCryptographicHash::Md5 );
        break;
    case 2:
        hasher->setAlgorithm( QCryptographicHash::Md4 );
        break;
    }

    ui->resultEdit->setText( QString("Working...") );
    hasher->start();
}
